@@ -1,41 +1,33 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 ** Copyright (C) 2012 Thorbjørn Lund Martsum - tmartsum[at]gmail.com
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
+** a written agreement between you and Digia. For licensing terms and
+** conditions see http://qt.digia.com/licensing. For further information
 ** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** rights. These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -368,6 +360,8 @@ void tst_QHeaderView::getSetCheck()
     QCOMPARE(0, obj1.minimumSectionSize());
     obj1.setMinimumSectionSize(99999);
     QCOMPARE(99999, obj1.minimumSectionSize());
+    obj1.setMinimumSectionSize(-1);
+    QVERIFY(obj1.minimumSectionSize() < 100);
 
     // int QHeaderView::offset()
     // void QHeaderView::setOffset(int)
@@ -619,6 +613,9 @@ void tst_QHeaderView::sectionSize_data()
 
 void tst_QHeaderView::sectionSize()
 {
+#if defined Q_OS_QNX
+    QSKIP("The section size is dpi dependent on QNX");
+#endif
     QFETCH(QList<int>, boundsCheck);
     QFETCH(QList<int>, defaultSizes);
     QFETCH(int, initialDefaultSize);
@@ -732,6 +729,9 @@ void tst_QHeaderView::visualIndexAt_data()
 
 void tst_QHeaderView::visualIndexAt()
 {
+#if defined Q_OS_QNX
+    QSKIP("The section size is dpi dependent on QNX");
+#endif
     QFETCH(QList<int>, hidden);
     QFETCH(QList<int>, from);
     QFETCH(QList<int>, to);
@@ -1645,6 +1645,38 @@ void tst_QHeaderView::saveRestore()
     QByteArray s2 = h2.saveState();
 
     QVERIFY(s1 == s2);
+    QVERIFY(!h2.restoreState(QByteArrayLiteral("Garbage")));
+
+    // QTBUG-40462
+    // Setting from Qt4, where information about multiple sections were grouped together in one
+    // sectionItem object
+    QByteArray settings_qt4 =
+      QByteArray::fromHex("000000ff00000000000000010000000100000000010000000000000000000000000000"
+                          "0000000003e80000000a0101000100000000000000000000000064ffffffff00000081"
+                          "0000000000000001000003e80000000a00000000");
+    QVERIFY(h2.restoreState(settings_qt4));
+    int sectionItemsLengthTotal = 0;
+    for (int i = 0; i < h2.count(); ++i)
+        sectionItemsLengthTotal += h2.sectionSize(i);
+    QVERIFY(sectionItemsLengthTotal == h2.length());
+
+    // Buggy setting where sum(sectionItems) != length. Check false is returned and this corrupted
+    // state isn't restored
+    QByteArray settings_buggy_length =
+      QByteArray::fromHex("000000ff000000000000000100000000000000050100000000000000000000000a4000"
+                          "000000010000000600000258000000fb0000000a010100010000000000000000000000"
+                          "0064ffffffff00000081000000000000000a000000d30000000100000000000000c800"
+                          "000001000000000000008000000001000000000000005c00000001000000000000003c"
+                          "0000000100000000000002580000000100000000000000000000000100000000000002"
+                          "580000000100000000000002580000000100000000000003c000000001000000000000"
+                          "03e8");
+    int old_length = h2.length();
+    QByteArray old_state = h2.saveState();
+    // Check setting is correctly recognized as corrupted
+    QVERIFY(!h2.restoreState(settings_buggy_length));
+    // Check nothing has been actually restored
+    QVERIFY(h2.length() == old_length);
+    QVERIFY(h2.saveState() == old_state);
 }
 
 void tst_QHeaderView::defaultSectionSizeTest()
