@@ -7,36 +7,28 @@
 **
 ** This file is part of the QtSerialPort module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
+** a written agreement between you and Digia. For licensing terms and
+** conditions see http://qt.digia.com/licensing. For further information
 ** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** rights. These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -51,6 +43,7 @@
 
 #include <initguid.h>
 #include <setupapi.h>
+#include <cfgmgr32.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -70,26 +63,36 @@ static inline const QList<GuidFlagsPair>& guidFlagsPairs()
     return guidFlagsPairList;
 }
 
+static QString toStringAndTrimNullCharacter(const QByteArray &buffer)
+{
+    QString result = QString::fromWCharArray(reinterpret_cast<const wchar_t *>(buffer.constData()),
+                                             buffer.size() / sizeof(wchar_t));
+    while (!result.isEmpty() && (result.at(result.size() - 1).unicode() == 0))
+        result.chop(1);
+    return result;
+}
+
 static QStringList portNamesFromHardwareDeviceMap()
 {
-    HKEY hKey = 0;
+    HKEY hKey = Q_NULLPTR;
     if (::RegOpenKeyEx(HKEY_LOCAL_MACHINE, L"HARDWARE\\DEVICEMAP\\SERIALCOMM", 0, KEY_QUERY_VALUE, &hKey) != ERROR_SUCCESS)
         return QStringList();
 
     QStringList result;
     DWORD index = 0;
     static const DWORD maximumValueNameInChars = 16383;
-    QByteArray outputValueName(maximumValueNameInChars * sizeof(wchar_t), 0);
+    QByteArray outputValueName;
+    outputValueName.resize(maximumValueNameInChars * sizeof(wchar_t));
     QByteArray outputBuffer;
     DWORD requiredDataBytes = 0;
     forever {
         DWORD requiredValueNameChars = maximumValueNameInChars;
         const LONG ret = ::RegEnumValue(hKey, index, reinterpret_cast<wchar_t *>(outputValueName.data()), &requiredValueNameChars,
-                                        NULL, NULL, reinterpret_cast<unsigned char *>(outputBuffer.data()), &requiredDataBytes);
+                                        Q_NULLPTR, Q_NULLPTR, reinterpret_cast<unsigned char *>(outputBuffer.data()), &requiredDataBytes);
         if (ret == ERROR_MORE_DATA) {
             outputBuffer.resize(requiredDataBytes);
         } else if (ret == ERROR_SUCCESS) {
-            result.append(QString::fromWCharArray(reinterpret_cast<const wchar_t *>(outputBuffer.constData())));
+            result.append(toStringAndTrimNullCharacter(outputBuffer));
             ++index;
         } else {
             break;
@@ -119,24 +122,37 @@ static QString deviceRegistryProperty(HDEVINFO deviceInfoSet,
         }
         devicePropertyByteArray.resize(requiredSize);
     }
-    return QString::fromWCharArray(reinterpret_cast<const wchar_t *>(devicePropertyByteArray.constData()));
+    return toStringAndTrimNullCharacter(devicePropertyByteArray);
 }
 
-static QString deviceInstanceIdentifier(HDEVINFO deviceInfoSet,
-                                        PSP_DEVINFO_DATA deviceInfoData)
+static QString deviceInstanceIdentifier(DEVINST deviceInstanceNumber)
 {
-    DWORD requiredSize = 0;
-    if (::SetupDiGetDeviceInstanceId(deviceInfoSet, deviceInfoData, NULL, 0, &requiredSize))
+    ULONG numberOfChars = 0;
+    if (::CM_Get_Device_ID_Size(&numberOfChars, deviceInstanceNumber, 0) != CR_SUCCESS)
         return QString();
-
-    QByteArray data(requiredSize * sizeof(wchar_t), 0);
-    if (!::SetupDiGetDeviceInstanceId(deviceInfoSet, deviceInfoData,
-                                      reinterpret_cast<wchar_t *>(data.data()), data.size(), NULL)) {
-        // TODO: error handling with GetLastError
+    // The size does not include the terminating null character.
+    ++numberOfChars;
+    QByteArray outputBuffer;
+    outputBuffer.resize(numberOfChars * sizeof(wchar_t));
+    if (::CM_Get_Device_ID(deviceInstanceNumber, reinterpret_cast<wchar_t *>(outputBuffer.data()),
+                           outputBuffer.size(), 0) != CR_SUCCESS) {
         return QString();
     }
+    return toStringAndTrimNullCharacter(outputBuffer).toUpper();
+}
 
-    return QString::fromWCharArray(reinterpret_cast<const wchar_t *>(data.constData()));
+static DEVINST parentDeviceInstanceNumber(DEVINST childDeviceInstanceNumber)
+{
+    ULONG nodeStatus = 0;
+    ULONG problemNumber = 0;
+    if (::CM_Get_DevNode_Status(&nodeStatus, &problemNumber,
+                                childDeviceInstanceNumber, 0) != CR_SUCCESS) {
+        return 0;
+    }
+    DEVINST parentInstanceNumber = 0;
+    if (::CM_Get_Parent(&parentInstanceNumber, childDeviceInstanceNumber, 0) != CR_SUCCESS)
+        return 0;
+    return parentInstanceNumber;
 }
 
 static QString devicePortName(HDEVINFO deviceInfoSet, PSP_DEVINFO_DATA deviceInfoData)
@@ -156,19 +172,22 @@ static QString devicePortName(HDEVINFO deviceInfoSet, PSP_DEVINFO_DATA deviceInf
         DWORD dataType = 0;
         QByteArray outputBuffer;
         forever {
-            const LONG ret = ::RegQueryValueEx(key, reinterpret_cast<const wchar_t *>(portNameKey.utf16()), NULL, &dataType,
+            const LONG ret = ::RegQueryValueEx(key, reinterpret_cast<const wchar_t *>(portNameKey.utf16()), Q_NULLPTR, &dataType,
                                                reinterpret_cast<unsigned char *>(outputBuffer.data()), &bytesRequired);
             if (ret == ERROR_MORE_DATA) {
                 outputBuffer.resize(bytesRequired);
                 continue;
             } else if (ret == ERROR_SUCCESS) {
                 if (dataType == REG_SZ)
-                    portName = QString::fromWCharArray((reinterpret_cast<const wchar_t *>(outputBuffer.constData())));
+                    portName = toStringAndTrimNullCharacter(outputBuffer);
                 else if (dataType == REG_DWORD)
                     portName = QStringLiteral("COM%1").arg(*(PDWORD(outputBuffer.constData())));
             }
             break;
         }
+
+        if (!portName.isEmpty())
+            break;
     }
     ::RegCloseKey(key);
     return portName;
@@ -191,7 +210,51 @@ private:
     const QString &m_serialPortName;
 };
 
-static QString deviceSerialNumber(const QString &instanceIdentifier)
+static QString deviceDescription(HDEVINFO deviceInfoSet,
+                                 PSP_DEVINFO_DATA deviceInfoData)
+{
+    return deviceRegistryProperty(deviceInfoSet, deviceInfoData, SPDRP_DEVICEDESC);
+}
+
+static QString deviceManufacturer(HDEVINFO deviceInfoSet,
+                                  PSP_DEVINFO_DATA deviceInfoData)
+{
+    return deviceRegistryProperty(deviceInfoSet, deviceInfoData, SPDRP_MFG);
+}
+
+static quint16 parseDeviceIdentifier(const QString &instanceIdentifier,
+                                     const QString &identifierPrefix,
+                                     int identifierSize, bool &ok)
+{
+    const int index = instanceIdentifier.indexOf(identifierPrefix);
+    if (index == -1)
+        return quint16(0);
+    return instanceIdentifier.mid(index + identifierPrefix.size(), identifierSize).toInt(&ok, 16);
+}
+
+static quint16 deviceVendorIdentifier(const QString &instanceIdentifier, bool &ok)
+{
+    static const int vendorIdentifierSize = 4;
+    quint16 result = parseDeviceIdentifier(
+                instanceIdentifier, QStringLiteral("VID_"), vendorIdentifierSize, ok);
+    if (!ok)
+        result = parseDeviceIdentifier(
+                    instanceIdentifier, QStringLiteral("VEN_"), vendorIdentifierSize, ok);
+    return result;
+}
+
+static quint16 deviceProductIdentifier(const QString &instanceIdentifier, bool &ok)
+{
+    static const int productIdentifierSize = 4;
+    quint16 result = parseDeviceIdentifier(
+                instanceIdentifier, QStringLiteral("PID_"), productIdentifierSize, ok);
+    if (!ok)
+        result = parseDeviceIdentifier(
+                    instanceIdentifier, QStringLiteral("DEV_"), productIdentifierSize, ok);
+    return result;
+}
+
+static QString parseDeviceSerialNumber(const QString &instanceIdentifier)
 {
     int firstbound = instanceIdentifier.lastIndexOf(QLatin1Char('\\'));
     int lastbound = instanceIdentifier.indexOf(QLatin1Char('_'), firstbound);
@@ -213,20 +276,24 @@ static QString deviceSerialNumber(const QString &instanceIdentifier)
     return instanceIdentifier.mid(firstbound + 1, lastbound - firstbound - 1);
 }
 
+static QString deviceSerialNumber(const QString &instanceIdentifier,
+                                  DEVINST deviceInstanceNumber)
+{
+    QString result = parseDeviceSerialNumber(instanceIdentifier);
+    if (result.isEmpty()) {
+        const DEVINST parentNumber = parentDeviceInstanceNumber(deviceInstanceNumber);
+        const QString parentInstanceIdentifier = deviceInstanceIdentifier(parentNumber);
+        result = parseDeviceSerialNumber(parentInstanceIdentifier);
+    }
+    return result;
+}
+
 QList<QSerialPortInfo> QSerialPortInfo::availablePorts()
 {
-    static const QString usbVendorIdentifierPrefix(QStringLiteral("VID_"));
-    static const QString usbProductIdentifierPrefix(QStringLiteral("PID_"));
-    static const QString pciVendorIdentifierPrefix(QStringLiteral("VEN_"));
-    static const QString pciDeviceIdentifierPrefix(QStringLiteral("DEV_"));
-
-    static const int vendorIdentifierSize = 4;
-    static const int productIdentifierSize = 4;
-
     QList<QSerialPortInfo> serialPortInfoList;
 
     foreach (const GuidFlagsPair &uniquePair, guidFlagsPairs()) {
-        const HDEVINFO deviceInfoSet = ::SetupDiGetClassDevs(reinterpret_cast<const GUID *>(&uniquePair.first), NULL, 0, uniquePair.second);
+        const HDEVINFO deviceInfoSet = ::SetupDiGetClassDevs(reinterpret_cast<const GUID *>(&uniquePair.first), Q_NULLPTR, Q_NULLPTR, uniquePair.second);
         if (deviceInfoSet == INVALID_HANDLE_VALUE)
             return serialPortInfoList;
 
@@ -236,51 +303,32 @@ QList<QSerialPortInfo> QSerialPortInfo::availablePorts()
 
         DWORD index = 0;
         while (::SetupDiEnumDeviceInfo(deviceInfoSet, index++, &deviceInfoData)) {
-            QSerialPortInfo serialPortInfo;
-
-            QString s = devicePortName(deviceInfoSet, &deviceInfoData);
-            if (s.isEmpty() || s.contains(QStringLiteral("LPT")))
+            const QString portName = devicePortName(deviceInfoSet, &deviceInfoData);
+            if (portName.isEmpty() || portName.contains(QStringLiteral("LPT")))
                 continue;
 
             if (std::find_if(serialPortInfoList.begin(), serialPortInfoList.end(),
-                             SerialPortNameEqualFunctor(s)) != serialPortInfoList.end()) {
+                             SerialPortNameEqualFunctor(portName)) != serialPortInfoList.end()) {
                 continue;
             }
 
-            serialPortInfo.d_ptr->portName = s;
-            serialPortInfo.d_ptr->device = QSerialPortPrivate::portNameToSystemLocation(s);
-            serialPortInfo.d_ptr->description =
-                    deviceRegistryProperty(deviceInfoSet, &deviceInfoData, SPDRP_DEVICEDESC);
-            serialPortInfo.d_ptr->manufacturer =
-                    deviceRegistryProperty(deviceInfoSet, &deviceInfoData, SPDRP_MFG);
+            QSerialPortInfoPrivate priv;
 
-            s = deviceInstanceIdentifier(deviceInfoSet, &deviceInfoData).toUpper();
+            priv.portName = portName;
+            priv.device = QSerialPortInfoPrivate::portNameToSystemLocation(portName);
+            priv.description = deviceDescription(deviceInfoSet, &deviceInfoData);
+            priv.manufacturer = deviceManufacturer(deviceInfoSet, &deviceInfoData);
 
-            serialPortInfo.d_ptr->serialNumber = deviceSerialNumber(s);
+            const QString instanceIdentifier = deviceInstanceIdentifier(deviceInfoData.DevInst);
 
-            int index = s.indexOf(usbVendorIdentifierPrefix);
-            if (index != -1) {
-                serialPortInfo.d_ptr->vendorIdentifier = s.mid(index + usbVendorIdentifierPrefix.size(), vendorIdentifierSize)
-                        .toInt(&serialPortInfo.d_ptr->hasVendorIdentifier, 16);
-            } else {
-                index = s.indexOf(pciVendorIdentifierPrefix);
-                if (index != -1)
-                    serialPortInfo.d_ptr->vendorIdentifier = s.mid(index + pciVendorIdentifierPrefix.size(), vendorIdentifierSize)
-                            .toInt(&serialPortInfo.d_ptr->hasVendorIdentifier, 16);
-            }
+            priv.serialNumber =
+                    deviceSerialNumber(instanceIdentifier, deviceInfoData.DevInst);
+            priv.vendorIdentifier =
+                    deviceVendorIdentifier(instanceIdentifier, priv.hasVendorIdentifier);
+            priv.productIdentifier =
+                    deviceProductIdentifier(instanceIdentifier, priv.hasProductIdentifier);
 
-            index = s.indexOf(usbProductIdentifierPrefix);
-            if (index != -1) {
-                serialPortInfo.d_ptr->productIdentifier = s.mid(index + usbProductIdentifierPrefix.size(), productIdentifierSize)
-                        .toInt(&serialPortInfo.d_ptr->hasProductIdentifier, 16);
-            } else {
-                index = s.indexOf(pciDeviceIdentifierPrefix);
-                if (index != -1)
-                    serialPortInfo.d_ptr->productIdentifier = s.mid(index + pciDeviceIdentifierPrefix.size(), productIdentifierSize)
-                            .toInt(&serialPortInfo.d_ptr->hasProductIdentifier, 16);
-            }
-
-            serialPortInfoList.append(serialPortInfo);
+            serialPortInfoList.append(priv);
         }
         ::SetupDiDestroyDeviceInfoList(deviceInfoSet);
     }
@@ -288,10 +336,10 @@ QList<QSerialPortInfo> QSerialPortInfo::availablePorts()
     foreach (const QString &portName, portNamesFromHardwareDeviceMap()) {
         if (std::find_if(serialPortInfoList.begin(), serialPortInfoList.end(),
                          SerialPortNameEqualFunctor(portName)) == serialPortInfoList.end()) {
-            QSerialPortInfo serialPortInfo;
-            serialPortInfo.d_ptr->portName = portName;
-            serialPortInfo.d_ptr->device = QSerialPortPrivate::portNameToSystemLocation(portName);
-            serialPortInfoList.append(serialPortInfo);
+            QSerialPortInfoPrivate priv;
+            priv.portName = portName;
+            priv.device =  QSerialPortInfoPrivate::portNameToSystemLocation(portName);
+            serialPortInfoList.append(priv);
         }
     }
 
@@ -306,7 +354,7 @@ QList<qint32> QSerialPortInfo::standardBaudRates()
 bool QSerialPortInfo::isBusy() const
 {
     const HANDLE handle = ::CreateFile(reinterpret_cast<const wchar_t*>(systemLocation().utf16()),
-                                           GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
+                                           GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, Q_NULLPTR);
 
     if (handle == INVALID_HANDLE_VALUE) {
         if (::GetLastError() == ERROR_ACCESS_DENIED)
@@ -320,7 +368,7 @@ bool QSerialPortInfo::isBusy() const
 bool QSerialPortInfo::isValid() const
 {
     const HANDLE handle = ::CreateFile(reinterpret_cast<const wchar_t*>(systemLocation().utf16()),
-                                           GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
+                                           GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, Q_NULLPTR);
 
     if (handle == INVALID_HANDLE_VALUE) {
         if (::GetLastError() != ERROR_ACCESS_DENIED)
@@ -331,4 +379,17 @@ bool QSerialPortInfo::isValid() const
     return true;
 }
 
+QString QSerialPortInfoPrivate::portNameToSystemLocation(const QString &source)
+{
+    return source.startsWith(QStringLiteral("COM"))
+            ? (QStringLiteral("\\\\.\\") + source) : source;
+}
+
+QString QSerialPortInfoPrivate::portNameFromSystemLocation(const QString &source)
+{
+    return (source.startsWith(QStringLiteral("\\\\.\\"))
+            || source.startsWith(QStringLiteral("//./")))
+            ? source.mid(4) : source;
+}
+
 QT_END_NAMESPACE
