@@ -1,11 +1,11 @@
 <?xml version="1.0" encoding="utf-8"?>
 <!DOCTYPE TS>
-<TS version="2.0" language="de_DE">
+<TS version="2.1" language="de_DE">
 <context>
-    <name>AudioCaptureSession</name>
+    <name>AudioContainerControl</name>
     <message>
-        <source>RAW file format</source>
-        <translation>RAW-Dateiformat</translation>
+        <source>RAW (headerless) file format</source>
+        <translation>RAW-Dateiformat (ohne Header)</translation>
     </message>
     <message>
         <source>WAV file format</source>
@@ -15,8 +15,8 @@
 <context>
     <name>AudioEncoderControl</name>
     <message>
-        <source>PCM audio data</source>
-        <translation>PCM-Audiodaten</translation>
+        <source>Linear PCM audio data</source>
+        <translation>PCM-Audiodaten (linear)</translation>
     </message>
 </context>
 <context>
@@ -159,6 +159,10 @@
         <source>Service has not been started</source>
         <translation>Der Dienst wurde nicht gestartet</translation>
     </message>
+    <message>
+        <source>Recording permissions are not available</source>
+        <translation>Berechtigung zur Aufnahme fehlt</translation>
+    </message>
 </context>
 <context>
     <name>CameraBinSession</name>
@@ -167,6 +171,17 @@
         <translation>Kamerafehler</translation>
     </message>
 </context>
+<context>
+    <name>DSCameraSession</name>
+    <message>
+        <source>Camera not ready for capture</source>
+        <translation>Die Kamera ist nicht aufnahmebereit</translation>
+    </message>
+    <message>
+        <source>Could not save image to file.</source>
+        <translation>Das Bild konnte nicht in eine Datei gespeichert werden.</translation>
+    </message>
+</context>
 <context>
     <name>MFPlayerSession</name>
     <message>
@@ -272,17 +287,81 @@
         <source>Error parsing playlist at line[%1]:%2</source>
         <translation>Fehler beim Auswerten der Abspielliste bei Zeile [%1]:%2</translation>
     </message>
+</context>
+<context>
+    <name>QAndroidAudioEncoderSettingsControl</name>
     <message>
-        <source>File%1</source>
-        <translation>Datei%1</translation>
+        <source>Adaptive Multi-Rate Narrowband (AMR-NB) audio codec</source>
+        <translation>Adaptive Multi-Rate Narrowband (AMR-NB) audio codec</translation>
     </message>
     <message>
-        <source>Title%1</source>
-        <translation>Titel%1</translation>
+        <source>Adaptive Multi-Rate Wideband (AMR-WB) audio codec</source>
+        <translation>Adaptive Multi-Rate Wideband (AMR-WB) audio codec</translation>
     </message>
     <message>
-        <source>Length%1</source>
-        <translation>Länge%1</translation>
+        <source>AAC Low Complexity (AAC-LC) audio codec</source>
+        <translation>AAC Low Complexity (AAC-LC) audio codec</translation>
+    </message>
+</context>
+<context>
+    <name>QAndroidCameraSession</name>
+    <message>
+        <source>Camera cannot be started without a viewfinder.</source>
+        <translation>Die Kamera kann nicht ohne Sucher gestartet werden.</translation>
+    </message>
+    <message>
+        <source>Camera not ready</source>
+        <translation>Kamera nicht bereit</translation>
+    </message>
+    <message>
+        <source>Drive mode not supported</source>
+        <extracomment>Drive mode is the camera&apos;s shutter mode, for example single shot, continuos exposure, etc.</extracomment>
+        <translation>Auslösemodus wird nicht unterstützt</translation>
+    </message>
+    <message>
+        <source>Could not open destination file: %1</source>
+        <translation>Die Zieldatei %1 konnte nicht geöffnet werden</translation>
+    </message>
+</context>
+<context>
+    <name>QAndroidImageEncoderControl</name>
+    <message>
+        <source>JPEG image</source>
+        <translation>JPEG-Bild</translation>
+    </message>
+</context>
+<context>
+    <name>QAndroidMediaContainerControl</name>
+    <message>
+        <source>MPEG4 media file format</source>
+        <translation>MPEG4-Medien-Dateiformat</translation>
+    </message>
+    <message>
+        <source>3GPP media file format</source>
+        <translation>3GPP-Medien-Dateiformat</translation>
+    </message>
+    <message>
+        <source>AMR NB file format</source>
+        <translation>AMR-NB-Dateiformat</translation>
+    </message>
+    <message>
+        <source>AMR WB file format</source>
+        <translation>AMR-WB-Dateiformat</translation>
+    </message>
+</context>
+<context>
+    <name>QAndroidVideoEncoderSettingsControl</name>
+    <message>
+        <source>H.263 compression</source>
+        <translation>H.263-Kompression</translation>
+    </message>
+    <message>
+        <source>H.264 compression</source>
+        <translation>H.264-Kompression</translation>
+    </message>
+    <message>
+        <source>MPEG-4 SP compression</source>
+        <translation>MPEG-4-SP-Kompression</translation>
     </message>
 </context>
 <context>
@@ -466,6 +545,10 @@
         <source>CameraImageProcessing is provided by Camera</source>
         <translation>CameraImageProcessing wird von Camera bereitgestellt</translation>
     </message>
+    <message>
+        <source>CameraViewfinder is provided by Camera</source>
+        <translation>CameraViewfinder wird von Camera bereitgestellt</translation>
+    </message>
 </context>
 <context>
     <name>QPlaylistFileParser</name>
@@ -489,4 +572,19 @@
         <translation>Leere Datei angegeben</translation>
     </message>
 </context>
+<context>
+    <name>QWinRTCameraImageCaptureControl</name>
+    <message>
+        <source>Camera not ready</source>
+        <translation>Kamera nicht bereit</translation>
+    </message>
+    <message>
+        <source>Invalid photo data length.</source>
+        <translation>Ungültige Länge der Bilddaten.</translation>
+    </message>
+    <message>
+        <source>Image saving failed</source>
+        <translation>Das Speichern des Bildes schlug fehl</translation>
+    </message>
+</context>
 </TS>
